'use strict';
const bcrypt = require('bcrypt');
const saltRounds = require(__dirname + '/../config/hash.json')['saltRounds'];
const https = require('https');
const parseString = require('xml2js').parseString;

const User = require('../models/index')['User'];
const Lecturer = require('../models/index')['Lecturer'];
const Student = require('../models/index')['Student'];

const { getJWTToken } = require('../util/jwt');
const { getUser } = require('../util/db');

const hashPassword = async function (req) {
  return new Promise((resolve) => {
    bcrypt.genSalt(saltRounds, (err, salt) => {
      bcrypt.hash(req.body.password, salt, (err2, hash) => {
        if (err2) console.log(err2);
        resolve({
          hashed: hash,
          salt: salt,
        });
      });
    });
  });
};

exports.register = async function (req, res) {
  let roles = ['student', 'lecturer'];
  let newPass = await hashPassword(req);
  if (roles.includes(req.body.role)) {
    const [user, created] = await User.findOrCreate({
      where: { email: req.body.email },
      defaults: { hashed_password: newPass.hashed, salt: newPass.salt },
    });

    if (!created) {
      console.log(user.id);
      if (req.body.role.toString() === 'student') {
        const [student, studentCreated] = await Student.findOrCreate({
          where: { userId: user.id },
        });
        if (!studentCreated) {
          console.log('Student with id:' + student.id + ' exist');
          res.status(400);
          res.json({ status: 400, message: 'Student exist' });
        } else {
          console.log('Student registered.');
          res.json({ status: 200, message: 'Register student successful.' });
        }
      } else {
        const [lecturer, lecturerCreated] = await Lecturer.findOrCreate({
          where: { userId: user.id },
        });
        if (!lecturerCreated) {
          console.log('Lecturer with id:' + lecturer.id + ' exist');
          res.status(400);
          res.json({ status: 400, message: 'Lecturer exist' });
        } else {
          console.log('Lecturer registered.');
          res.json({ status: 200, message: 'Register lecturer successful.' });
        }
      }
    } else {
      if (req.body.role.toString() === 'student') {
        const student = await Student.create({ userId: user.id });
        if (student) {
          console.log('Student registered.');
          res.json({ status: 200, message: 'Register student successful.' });
        } else {
          console.log('Student creation failed.');
          res.status(500);
          res.json({ status: 500, message: 'Register student failed.' });
        }
      } else {
        const lecturer = await Lecturer.create({ userId: user.id });
        if (lecturer) {
          console.log('Lecturer registered.');
          res.json({ status: 200, message: 'Register lecturer successful.' });
        } else {
          console.log('Lecturer creation failed.');
          res.status(500);
          res.json({ status: 500, message: 'Register lecturer failed.' });
        }
      }
    }
  } else {
    res.status(400);
    res.json({
      status: 400,
      message: 'Role undefined',
    });
  }
};

exports.verifyUser = async (req, res) => {
  const { email, password } = req.body;

  try {
    const user = await getUser(email, password);
    const { id, email: userEmail, role } = user;

    const tokenElement = {
      id,
      email: userEmail,
      role,
    };

    const { COOKIE_NAME } = process.env;

    const token = getJWTToken(tokenElement);

    const cookieOpts = {
      secure: true,
      httpOnly: true,
    };

    res.cookie(COOKIE_NAME, token, cookieOpts);
    res.json(tokenElement);
  } catch (error) {
    console.error(error);

    res.status(401);
    res.json({
      message: error.message,
    });
  }
};

exports.casSSO = async (req, res) => {
  if (req.query.ticket) {
    https
      .get(
        'https://login.itb.ac.id/cas/serviceValidate?ticket=' +
          req.query.ticket +
          '&service=' +
          req.query.service,
        (resp) => {
          let data = '';

          resp.on('data', (chunk) => {
            parseString(chunk, function (err, result) {
              data = JSON.parse(JSON.stringify(result).replace(/cas:/g, ''));
            });
          });

          resp.on('end', () => {
            let processedData = {};

            if (data.serviceResponse.authenticationSuccess) {
              let successData = data.serviceResponse.authenticationSuccess[0];

              processedData = {
                username: successData.user[0],
                email: successData.attributes[0].mail[0],
                nim: successData.attributes[0].itbNIM[0],
                prodi: successData.attributes[0].ou[0],
                fullName: successData.attributes[0].cn[0],
                status: successData.attributes[0].itbStatus[0],
              };
            } else {
              processedData = {
                username: null,
              };
            }

            res.json(processedData);
          });
        },
      )
      .on('error', (err) => {
        console.log('Error: ' + err.message);
      });
  } else {
    res.json('Invalid ticket');
  }
};
