'use strict';
const {
  handleRequestWithInvalidRequestBody,
  checkRequiredParameter,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
} = require('../util/common');

const { CourseClassMeetingAttendance, Student } = require('../models/index');

const {
  RequiredParameterUndefinedError,
  NotExistError,
} = require('../util/error');
const {
  getCourseClassMeetingById,
} = require('../util/db/course-class-meeting');

const getCourseClassMeetingMiddleware = async (req, res, next) => {
  const {
    courseClassMeetingId,
    studentId,
    lecturerId,
    startYear,
    semester,
    shouldIncludeAttendances = 'false',
  } = req.query;

  try {
    const opts = {
      where: {},
    };

    checkRequiredParameter({
      oneOf: {
        courseClassMeetingId,
        studentId,
        lecturerId,
      },
    });

    if (courseClassMeetingId) {
      opts.where.id = courseClassMeetingId;
      if (shouldIncludeAttendances.toLowerCase() === 'true') {
        opts.include = [
          {
            model: CourseClassMeetingAttendance,
            as: 'courseClassMeetingAttendances',
            attributes: ['status'],
            include: Student,
          },
        ];
      }
    } else if (studentId) {
      checkRequiredParameter({
        semester,
        startYear,
      });

      opts.where = {
        studentId,
        semester,
        startYear,
      };
      opts.include = 'studyPlanCourses';
    } else if (lecturerId) {
      checkRequiredParameter({
        semester,
        startYear,
      });

      opts.where = {
        lecturerId,
        semester,
        startYear,
      };
    }

    req.body.opts = opts;

    next();
  } catch (error) {
    if (error instanceof RequiredParameterUndefinedError)
      handleRequestWithInvalidRequestBody(res, error);
    else handleRequestWithInternalServerError(res, error);
  }
};

const updateCourseClassMeetingBodyCheckerMiddleware = async (
  req,
  res,
  next,
) => {
  try {
    const {
      startClass,
      endClass,
      startAttendance,
      endAttendance,
      link,
      topic,
      notes,
      attendances,
    } = req.body;

    checkRequiredParameter({
      oneOf: {
        startClass,
        endClass,
        startAttendance,
        endAttendance,
        link,
        topic,
        notes,
        attendances,
      },
    });

    next();
  } catch (error) {
    if (error instanceof RequiredParameterUndefinedError) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

const updateCourseClassMeetingMiddleware = async (req, res, next) => {
  const { id } = req.params;

  try {
    const courseClassMeeting = await getCourseClassMeetingById({
      where: {
        id,
      },
      shouldIncludeAttendances: true,
    });

    const {
      startClass,
      endClass,
      startAttendance,
      endAttendance,
      link,
      topic,
      notes,
      attendances,
    } = req.body;

    const changeset = {
      startTime: startClass,
      endTime: endClass,
      openAttendanceTime: startAttendance,
      closeAttendanceTime: endAttendance,
      link,
      topic,
      notes,
      courseClassMeetingAttendances: attendances,
    };

    req.body.courseClassMeeting = courseClassMeeting;
    req.body.changeset = changeset;

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

module.exports = {
  getCourseClassMeetingMiddleware,
  updateCourseClassMeetingBodyCheckerMiddleware,
  updateCourseClassMeetingMiddleware,
};
