'use strict';

const {
  Course,
  Major,
  Faculty,
  Lecturer,
  CourseClassDefaultSchedule,
} = require('../models/index');

const {
  handleRequestWithInvalidRequestBody,
  checkRequiredParameter,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
} = require('../util/common');
const { getCourseClass } = require('../util/db/course-class');
const {
  RequiredParameterUndefinedError,
  NotExistError,
} = require('../util/error');

const getCourseClassMiddleware = async (req, res, next) => {
  const { status, semester, startYear, include } = req.query;

  const { id } = req.params;

  let opts = {};

  if (id) {
    opts.where = { id };
  } else {
    opts.where = {};
    if (status) {
      opts.where.status = status;
    }

    if (semester) {
      opts.where.semester = semester;
    }

    if (startYear) {
      opts.where.startYear = startYear;
    }
  }

  opts.include = [
    {
      model: CourseClassDefaultSchedule,
      as: 'courseClassDefaultSchedules',
    },
  ];

  if (include) {
    if (include.includes('Course')) {
      opts.include.push({
        model: Course,
        include: [
          {
            model: Major,
            include: Faculty,
          },
        ],
      });
    }

    if (include.includes('Lecturer')) {
      opts.include.push({
        model: Lecturer,
      });
    }

    if (include.includes('Students')) {
      req.body.includeStudent = true;
    }
  }

  req.body.opts = opts;

  next();
};

const updateCourseClassMiddleware = async (req, res, next) => {
  const { id } = req.params;

  try {
    const {
      status,
      startYear,
      semester,
      capacity,
      lecturerId,
      courseClassDefaultSchedules,
    } = req.body;

    checkRequiredParameter({
      oneOf: {
        status,
        startYear,
        semester,
        capacity,
        lecturerId,
        courseClassDefaultSchedules,
      },
    });

    const courseClass = await getCourseClass({
      body: {
        opts: {
          where: {
            id,
          },
          include: [
            {
              model: CourseClassDefaultSchedule,
              as: 'courseClassDefaultSchedules',
            },
          ],
        },
      },
    });

    const changeset = {};

    if (status) {
      changeset.status = status;
    }

    if (lecturerId) {
      changeset.lecturerId = lecturerId;
    }

    if (startYear) {
      changeset.startYear = startYear;
    }

    if (semester) {
      changeset.semester = semester;
    }

    if (capacity) {
      changeset.capacity = capacity;
    }

    if (courseClassDefaultSchedules) {
      changeset.courseClassDefaultSchedules = courseClassDefaultSchedules;
    }

    req.body.courseClass = courseClass;
    req.body.changeset = changeset;

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else if (error instanceof RequiredParameterUndefinedError) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

module.exports = {
  getCourseClassMiddleware,
  updateCourseClassMiddleware,
};
