const {
  handleRequestWithInvalidRequestBody,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
  checkRequiredParameter
} = require('../util/common');

const {
  RequiredParameterUndefinedError,
  NotExistError
} = require('../util/error');

const { getOption } = require('../util/db/option');
const { getCourse } = require('../util/db/course');

const {
  CourseS2,
  Major,
  Faculty
} = require('../models/index');

const createCourseMiddleware = async (req, res, next) => {
  let newCourse;

  let {
    previousId,
    code,
    name,
    type,
    credits,
    defaultSemester,
    shortSyllabus,
    completeSyllabus,
    outcomes,
    curriculumYear,
    majorId,
    optionId
  } = req.body;


  try {
    if (previousId) {
        checkRequiredParameter({curriculumYear});
        const previousCourse = await getCourse({
          where: {
            id: previousId
          },
          include: [{
            model: CourseS2,
            as: 'courseS2'
          }]
        });
      code = previousCourse.code;
      name = previousCourse.name;
      type = previousCourse.type;
      credits = previousCourse.credits;
      defaultSemester = previousCourse.defaultSemester;
      shortSyllabus = previousCourse.shortSyllabus;
      completeSyllabus = previousCourse.completeSyllabus;
      outcomes = previousCourse.outcomes;
      majorId = previousCourse.majorId;
      optionId = previousCourse.courseS2[0].optionId;
    }

    checkRequiredParameter({ code, name, type, credits, curriculumYear });
    checkRequiredParameter({
      oneOf: {
        majorId,
        optionId
      }
    });

    if (optionId){
      const option = await getOption({
                        where: {
                          id: optionId
                        }
                      });
      majorId = option.majorId;
    }

    newCourse = {
      code,
      name,
      type,
      credits,
      defaultSemester,
      shortSyllabus,
      completeSyllabus,
      outcomes,
      curriculumYear,
      majorId,
    };

    if (optionId) {
      newCourse.courseS2 = {
        optionId
      };
    }

    req.body.newCourse = newCourse;

    next();
  } catch (error) {
    if (error instanceof RequiredParameterUndefinedError)
      handleRequestWithInvalidRequestBody(res, error);
    else handleRequestWithInternalServerError(res, error);
  }
};

const updateCourseMiddleware = async (req, res, next) => {
  const { id } = req.params;

  try {
    const {
      code,
      name,
      type,
      credits,
      defaultSemester,
      shortSyllabus,
      completeSyllabus,
      outcomes,
      curriculumYear,
    } = req.body;

    checkRequiredParameter({
      oneOf: {
        code,
        name,
        type,
        credits,
        defaultSemester,
        shortSyllabus,
        completeSyllabus,
        outcomes,
        curriculumYear,
      },
    });

    const course = await getCourse({
      where: {
        id,
      }
    });

    const changeset = {};

    if (code) {
      changeset.code = code;
    }

    if (name) {
      changeset.name = name;
    }

    if (type) {
      changeset.type = type;
    }

    if (defaultSemester) {
      changeset.defaultSemester = defaultSemester;
    }

    if (curriculumYear) {
      changeset.curriculumYear = curriculumYear;
    }

    if (credits) {
      changeset.credits = credits;
    }

    if (shortSyllabus) {
      changeset.shortSyllabus = shortSyllabus;
    }

    if (completeSyllabus) {
      changeset.completeSyllabus = completeSyllabus;
    }

    if (outcomes) {
      changeset.outcomes = outcomes;
    }

    req.body.course = course;
    req.body.changeset = changeset;

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else if (error instanceof RequiredParameterUndefinedError) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};


const getAllCoursesMiddleware = async (req, res, next) => {
  const {
    facultyId,
    majorId,
    curriculumYear
  } = req.query;

  let opts = {
    where: {},
    include: []
  };

  if (curriculumYear) {
    opts.where.curriculumYear = curriculumYear;
  }

  if (facultyId) {
    opts.include = [{
      model: Major,
      include:[{
        model: Faculty,
        where: {
          id: facultyId
        }
      }]
    }];
  }

  if (majorId) {
    opts.include = [{
      model: Major,
      where: {
        id: majorId
      }
    }];
  }

  opts.include.push(
    {
      model: CourseS2,
      as: 'courseS2',
    },
  );

  req.body.opts = opts;

  next();
};

module.exports = {
  createCourseMiddleware,
  updateCourseMiddleware,
  getAllCoursesMiddleware
}