'use strict';
const {
  handleRequestWithInvalidRequestBody,
  checkRequiredParameter,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
} = require('../util/common');

const {
  getStudyPlanCourse,
  getBulkStudyPlanCourses,
} = require('../util/db/study-plan');

const {
  RequiredParameterUndefinedError,
  NotExistError,
} = require('../util/error');
const { StudyPlan } = require('../models/index');

const updateScoreMiddleware = async (req, res, next) => {
  try {
    const { studentId, courseClassId } = req.query;
    const { score } = req.body;

    checkRequiredParameter({
      studentId,
      courseClassId,
      score,
    });

    const studyPlanCourse = await getStudyPlanCourse({
      where: {
        courseClassId,
      },
      include: {
        model: StudyPlan,
        where: {
          studentId,
        },
      },
    });

    const changeset = {
      score,
    };

    req.body = {
      ...req.body,
      studyPlanCourse,
      changeset,
    };

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else if (error instanceof RequiredParameterUndefinedError) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

const updateScoreStatusMiddleware = async (req, res, next) => {
  try {
    const { courseClassId } = req.query;
    const { scoreStatus } = req.body;

    checkRequiredParameter({
      courseClassId,
      scoreStatus,
    });

    const studyPlanCourses = await getBulkStudyPlanCourses({
      where: {
        courseClassId,
      },
    });

    if (studyPlanCourses.length === 0) throw new NotExistError();

    const changeset = {
      scoreStatus,
    };

    req.body = {
      ...req.body,
      studyPlanCourses,
      changeset,
    };

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else if (error instanceof RequiredParameterUndefinedError) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

module.exports = {
  updateScoreMiddleware,
  updateScoreStatusMiddleware,
};
