'use strict';
const {
  handleRequestWithInvalidRequestBody,
  checkRequiredParameter,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
} = require('../util/common');
const { getStudyPlan } = require('../util/db/study-plan');
const {
  RequiredParameterUndefinedError,
  NotExistError,
} = require('../util/error');
const {
  checkClassAvailability
} = require('../util/hook');

const getStudyPlanMiddleware = async (req, res, next) => {
  const {
    studyPlanId,
    studentId,
    semester,
    startYear,
    shouldIncludeCourses = 'true',
  } = req.query;

  try {
    const opts = {
      where: {},
    };

    checkRequiredParameter({
      oneOf: {
        studentId,
        studyPlanId,
      },
    });

    if (studyPlanId) {
      opts.where.id = studyPlanId;
    } else if (studentId) {
      checkRequiredParameter({
        studentId,
        semester,
        startYear,
      });

      opts.where = {
        studentId,
        semester,
        startYear,
      };
    }

    if (shouldIncludeCourses.toLowerCase() === 'true') {
      opts.include = 'studyPlanCourses';
    }

    req.body.opts = opts;

    next();
  } catch (error) {
    if (error instanceof RequiredParameterUndefinedError)
      handleRequestWithInvalidRequestBody(res, error);
    else handleRequestWithInternalServerError(res, error);
  }
};

const updateStudyPlanMiddleware = async (req, res, next) => {
  const { id } = req.params;

  try {
    const { status, notes, studyPlanCourses } = req.body;

    checkRequiredParameter({
      oneOf: {
        status,
        notes,
        studyPlanCourses,
      },
    });

    const studyPlan = await getStudyPlan({
      where: {
        id,
      },
      include: 'studyPlanCourses',
    });

    const changeset = {};

    if (status) {
      changeset.status = status;
    }

    if (notes) {
      changeset.notes = notes;
    }

    if (studyPlanCourses) {
      await checkClassAvailability(studyPlan.semester, studyPlan.startYear, studyPlanCourses);
      changeset.studyPlanCourses = studyPlanCourses;
    }

    req.body.studyPlan = studyPlan;
    req.body.changeset = changeset;

    next();
  } catch (error) {
    if (error instanceof NotExistError) {
      handleRequestWithResourceItemNotFound(res, error);
    } else if (
      error instanceof RequiredParameterUndefinedError ||
      error instanceof ClassUnavailableError
    ) {
      handleRequestWithInvalidRequestBody(res, error);
    } else {
      handleRequestWithInternalServerError(res, error);
    }
  }
};

module.exports = {
  getStudyPlanMiddleware,
  updateStudyPlanMiddleware,
};
