const {
  Course,
  CourseS2,
  Faculty,
  Major,
  Option
}= require('../models/index');

let chai = require('chai');
let chaiHttp = require('chai-http');
let server = require('../index');
let should = chai.should();

chai.use(chaiHttp);

const {
  EndpointEnum,
  CourseTypeEnum
} = require('../enums/index');

const CODE = 'IF5122';
const NAME = 'Pembangunan Perangkat Lunak';
const DEFAULT_SEMESTER = 1;
const TYPE = CourseTypeEnum.MANDATORY;
const CURRICULUM_YEAR = 2013;
const NEXT_CURRICULUM_YEAR = 2019;
const OUTCOMES = 'Outcome';
const SHORT_SYLLABUS = 'Short syllabus';
const COMPLETE_SYLLABUS = 'Complete syllabus';
const CREDITS = 2;
const FACULTY_NAME = 'STEI';
let DUMMY_ID = '57e19842-d712-45fd-9068-04cde6bf41f3';
let INVALID_ID = '57e19842-d712-45fd-9068-04cde6bf41f';

const REQUEST_ERROR_CODE = 400;
const OBJECT_NOT_EXIST_ERROR_CODE = 404;
const SUCCESS_CODE = 200;
const SERVER_ERROR_CODE = 500;
const COURSE_DOESNT_EXIST_ERROR = "Object doesn't exist.";
const ERROR = 'Error';
const INVALID_UUID_ERROR =
  'invalid input syntax for type uuid: "' + INVALID_ID + '"';
const SEQUELIZE_DATABASE_ERROR = 'SequelizeDatabaseError';
const TYPE_UNDEFINED_ERROR = 'Undefined type';
const REQUIRED_PARAMETERES_UNDEFINED_ERROR = 'Undefined one of these parameters: code, or name, or type, or credits, or defaultSemester, or shortSyllabus, or completeSyllabus, or outcomes, or curriculumYear';

describe('Course Test', () => {
  let major, option, course, courseS2;

  beforeEach(async () => {
    await Course.destroy({ where: {} });
    await CourseS2.destroy({ where: {} });
    await Faculty.destroy({ where: {} });
    await Major.destroy({ where: {} });
    await Option.destroy({ where: {} });

    const faculty = await Faculty.create({
      shortName: FACULTY_NAME
    });

    major = await Major.create({
      facultyId: faculty.id
    });

    option = await Option.create({
      majorId: major.id
    });

    course = await Course.create({
      code: CODE,
      name: NAME,
      type: TYPE,
      credits: CREDITS,
      majorId: major.id,
      curriculumYear: CURRICULUM_YEAR
    });

    courseS2 = await CourseS2.create({
      courseId: course.id,
      optionId: option.id
    })
  });

  afterEach(() => {
    server.close();
  });

  describe('/GET all course', () => {
    beforeEach(async () => {
      await Course.create({
        code: CODE,
        name: NAME,
        curriculumYear: NEXT_CURRICULUM_YEAR
      });
    });

    it('should get all courses data', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.COURSE)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('code', CODE);
          chai.expect(res.body[0]).to.have.property('name', NAME);
          done();
        });
    });
  });

  describe('/GET/:id course', () => {
    let course;

    beforeEach(async () => {
      course = await Course.create({
        code: CODE,
        name: NAME,
        curriculumYear: NEXT_CURRICULUM_YEAR
      });
    });

    it('course get by id success', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.COURSE + '/' + course.id)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('code');
          res.body.should.have.property('name');
          res.body.should.have.property('type');
          res.body.should.have.property('credits');
          res.body.should.have.property('defaultSemester');
          res.body.should.have.property('shortSyllabus');
          res.body.should.have.property('completeSyllabus');
          res.body.should.have.property('outcomes');
          res.body.should.have.property('curriculumYear');
          res.body.should.have.property('majorId');
          res.body.should.have.property('Major');
          done();
        });
    });

    it("course get by id doesn't exist", (done) => {
      chai
        .request(server)
        .get(EndpointEnum.COURSE + '/' + DUMMY_ID)
        .end((err, res) => {
          res.should.have.status(OBJECT_NOT_EXIST_ERROR_CODE);
          res.body.should.have
            .property('name')
            .eql(ERROR);
          res.body.should.have
            .property('message')
            .eql(COURSE_DOESNT_EXIST_ERROR);
          done();
        });
    });

    it('course get by id invalid uuid', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.COURSE + '/' + INVALID_ID)
        .end((err, res) => {
          res.should.have.status(REQUEST_ERROR_CODE);
          res.body.should.have.property('name').eql(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have.property('message').eql(INVALID_UUID_ERROR);
          done();
        });
    });
  });

  describe('Creating course data', () => {
    it('should return course data when request body S2 valid', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        type: TYPE,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: NEXT_CURRICULUM_YEAR,
        optionId: option.id
      };

      chai
        .request(server)
        .post(EndpointEnum.COURSE)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('code').deep.equal(requestBody.code);
          res.body.should.have.property('name').deep.equal(requestBody.name);
          res.body.should.have
            .property('type')
            .deep.equal(requestBody.type);
          res.body.should.have
            .property('credits')
            .deep.equal(requestBody.credits);
          res.body.should.have
            .property('defaultSemester')
            .deep.equal(requestBody.defaultSemester);
          res.body.should.have
            .property('curriculumYear')
            .deep.equal(requestBody.curriculumYear);
          res.body.should.have
            .property('shortSyllabus')
            .deep.equal(requestBody.shortSyllabus);
          res.body.should.have
            .property('completeSyllabus')
            .deep.equal(requestBody.completeSyllabus);
          res.body.should.have
            .property('outcomes')
            .deep.equal(requestBody.outcomes);
          res.body.should.have.property('courseS2').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return course data when request body S1 valid', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        type: TYPE,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: NEXT_CURRICULUM_YEAR,
        majorId: major.id
      };

      chai
        .request(server)
        .post(EndpointEnum.COURSE)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('code').deep.equal(requestBody.code);
          res.body.should.have.property('name').deep.equal(requestBody.name);
          res.body.should.have
            .property('type')
            .deep.equal(requestBody.type);
          res.body.should.have
            .property('credits')
            .deep.equal(requestBody.credits);
          res.body.should.have
            .property('defaultSemester')
            .deep.equal(requestBody.defaultSemester);
          res.body.should.have
            .property('curriculumYear')
            .deep.equal(requestBody.curriculumYear);
          res.body.should.have
            .property('shortSyllabus')
            .deep.equal(requestBody.shortSyllabus);
          res.body.should.have
            .property('completeSyllabus')
            .deep.equal(requestBody.completeSyllabus);
          res.body.should.have
            .property('outcomes')
            .deep.equal(requestBody.outcomes);
          res.body.should.have
            .property('majorId')
            .deep.equal(requestBody.majorId);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return course data when import previous data request body valid', (done) => {
      const requestBody = {
        previousId: course.id,
        curriculumYear: NEXT_CURRICULUM_YEAR,
      };

      chai
        .request(server)
        .post(EndpointEnum.COURSE)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('code').deep.equal(CODE);
          res.body.should.have.property('name').deep.equal(NAME);
          res.body.should.have
            .property('type')
            .deep.equal(TYPE);
          res.body.should.have
            .property('credits')
            .deep.equal(CREDITS);
          res.body.should.have.property('defaultSemester');
          res.body.should.have
            .property('curriculumYear')
            .deep.equal(requestBody.curriculumYear);
          res.body.should.have.property('shortSyllabus');
          res.body.should.have.property('completeSyllabus');
          res.body.should.have.property('outcomes');
          res.body.should.have.property('majorId').to.not.equal(null);
          res.body.should.have.property('courseS2').to.not.equal(null);
          chai.expect(res.body.courseS2).to.be.an('array');
          chai.expect(res.body.courseS2[0]).to.have.property('optionId', option.id);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return error when request body invalid (wrong course type enum)', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        type: `${TYPE}A`,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: NEXT_CURRICULUM_YEAR,
        majorId: major.id
      };

      chai
        .request(server)
        .post(EndpointEnum.COURSE)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SERVER_ERROR_CODE);
          res.body.should.have
            .property('name')
            .to.equal(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
            'invalid input value for enum "enum_Courses_type": "WAJIBA"',
          );
          done();
        });
    });

    it('should return error when request body invalid (required parameter type not given)', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: CURRICULUM_YEAR,
        majorId: major.id
      };

      chai
        .request(server)
        .post(EndpointEnum.COURSE)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR_CODE);
          res.body.should.have
            .property('name')
            .to.equal(ERROR);
          res.body.should.have
            .property('message')
            .to.equal(TYPE_UNDEFINED_ERROR);
          done();
        });
    });
  });


  describe('Updating course data', () => {
    it('should return course data when update request body valid', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        type: TYPE,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: CURRICULUM_YEAR,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE}/${course.id}`)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('code').deep.equal(requestBody.code);
          res.body.should.have.property('name').deep.equal(requestBody.name);
          res.body.should.have
            .property('type')
            .deep.equal(requestBody.type);
          res.body.should.have
            .property('credits')
            .deep.equal(requestBody.credits);
          res.body.should.have
            .property('defaultSemester')
            .deep.equal(requestBody.defaultSemester);
          res.body.should.have
            .property('curriculumYear')
            .deep.equal(requestBody.curriculumYear);
          res.body.should.have
            .property('shortSyllabus')
            .deep.equal(requestBody.shortSyllabus);
          res.body.should.have
            .property('completeSyllabus')
            .deep.equal(requestBody.completeSyllabus);
          res.body.should.have
            .property('outcomes')
            .deep.equal(requestBody.outcomes);
          res.body.should.have.property('majorId').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return error when request body invalid (wrong type enum)', (done) => {
      const requestBody = {
        code: CODE,
        name: NAME,
        type: `${TYPE}A`,
        credits: CREDITS,
        defaultSemester: DEFAULT_SEMESTER,
        shortSyllabus: SHORT_SYLLABUS,
        completeSyllabus: COMPLETE_SYLLABUS,
        outcomes: OUTCOMES,
        curriculumYear: CURRICULUM_YEAR,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE}/${course.id}`)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SERVER_ERROR_CODE);
          res.body.should.have
            .property('name')
            .to.equal(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
            'invalid input value for enum "enum_Courses_type": "WAJIBA"',
          );
          done();
        });
    });

    it('should return error when request body invalid (required parameter type not given)', (done) => {
      const requestBody = {};

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE}/${course.id}`)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR_CODE);
          res.body.should.have
            .property('name')
            .to.equal(ERROR);
          res.body.should.have
            .property('message')
            .to.equal(REQUIRED_PARAMETERES_UNDEFINED_ERROR);
          done();
        });
    });
  });
});
