const {
  CourseClass,
  Course,
  User,
  Lecturer,
  Student,
  StudyPlan,
  StudyPlanCourse,
} = require('../models/index');

const {
  EndpointEnum,
  DayEnum,
  StudyPlanCourseStatusEnum,
  SemesterEnum,
  StudyPlanStatusEnum,
} = require('../enums/index');

const chai = require('chai');
const chaiHttp = require('chai-http');
const server = require('../index');
const should = chai.should();

chai.use(chaiHttp);

const EMAIL = 'ihsan@gmail.com';
const PASSWORD = 'abc123';
const SALT = 'salt';
const STATUS = 'OPEN';
const START_YEAR = 2021;
const NEW_START_YEAR = 2022;
const CAPACITY = 20;
const NEW_CAPACITY = 50;
const COURSE_CODE = 'IF2036';
const COURSE_NAME = 'Rekayasa Perangkat Lunak';
const DUMMY_ID = '57e19842-d712-45fd-9068-04cde6bf41f3';
const INVALID_ID = '57e19842-d712-45fd-9068-04cde6bf41f';
const START_TIME = '13:00:00';
const END_TIME = '15:00:00';

const COURSE = 'Course';
const LECTURER = 'Lecturer';
const STUDENTS = 'Students';

const COURSE_CLASS_ENDPOINT = '/course-class';

const SEQUELIZE_DATABASE_ERROR = 'SequelizeDatabaseError';
const SEQUELIZE_FOREIGN_KEY_ERROR = 'SequelizeForeignKeyConstraintError';
const SUCCESS_CODE = 200;
const REQUEST_ERROR_CODE = 400;
const NOT_EXIST_ERROR_CODE = 404;
const SERVER_ERROR_CODE = 500;
const ERROR = 'Error';

describe('Course Class Test', () => {
  let user, course, lecturer, student, courseClass;

  beforeEach(async () => {
    await CourseClass.destroy({ where: {} });
    await Course.destroy({ where: {} });
    await Lecturer.destroy({ where: {} });
    await Student.destroy({ where: {} });
    await StudyPlan.destroy({ where: {} });
    await StudyPlanCourse.destroy({ where: {} });

    course = await Course.create({
      code: COURSE_CODE,
      name: COURSE_NAME,
    });

    user = await User.create({
      email: EMAIL,
      hashed_password: PASSWORD,
      salt: SALT,
    });

    lecturer = await Lecturer.create({
      userId: user.id,
    });

    courseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: STATUS,
      capacity: CAPACITY,
      courseId: course.id,
      lecturerId: lecturer.id,
    });
    student = await Student.create({
      userId: user.id,
    });

    const studyPlan = await StudyPlan.create({
      studentId: student.id,
      status: StudyPlanStatusEnum.FINAL
    });

    await StudyPlanCourse.create({
      studyPlanId: studyPlan.id,
      courseClassId: courseClass.id,
      status: StudyPlanCourseStatusEnum.APPROVED,
    });
  });

  afterEach(() => {
    server.close();
  });

  describe('/GET course class', () => {
    it('should get filtered course classes data', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${COURSE}&include=${LECTURER}`,
        )
        .end((err, res) => {
          res.should.have.status(200);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('startYear', START_YEAR);
          chai.expect(res.body[0]).to.have.property('status', STATUS);
          chai.expect(res.body[0]).to.have.property('capacity', CAPACITY);
          chai.expect(res.body[0]).to.have.property('courseId', course.id);
          chai.expect(res.body[0]).to.have.property('lecturerId', lecturer.id);
          chai.expect(res.body[0]).to.have.property('Course');
          chai.expect(res.body[0]).to.have.property('Lecturer');
          done();
        });
    });

    it('should get all course classes data include participant', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${STUDENTS}`,
        )
        .end((err, res) => {
          res.should.have.status(200);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('startYear', START_YEAR);
          chai.expect(res.body[0]).to.have.property('status', STATUS);
          chai.expect(res.body[0]).to.have.property('capacity', CAPACITY);
          chai.expect(res.body[0]).to.have.property('courseId', course.id);
          chai.expect(res.body[0]).to.have.property('lecturerId', lecturer.id);
          chai.expect(res.body[0]).to.have.property('students');
          chai
            .expect(res.body[0].students[0])
            .to.have.property('id')
            .eql(student.id);
          done();
        });
    });

    it('course class get filter not exist', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/?status=${STATUS}&semester=${SemesterEnum.EVEN}&startYear=${START_YEAR}&include=${STUDENTS}`,
        )
        .end((err, res) => {
          res.should.have.status(NOT_EXIST_ERROR_CODE);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });
  });

  describe('/GET/:id lecturer', () => {
    it('course class get by id success', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/${courseClass.id}?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${COURSE}&include=${LECTURER}`,
        )
        .end((err, res) => {
          res.should.have.status(200);
          res.body.should.have.property('id');
          res.body.should.have.property('startYear');
          res.body.should.have.property('capacity');
          res.body.should.have.property('participantCount');
          res.body.should.have.property('semester');
          res.body.should.have.property('status');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('courseId');
          res.body.should.have.property('lecturerId');
          res.body.should.have.property('Course');
          res.body.should.have.property('Lecturer');
          res.body.should.have.property('courseClassDefaultSchedules');
          done();
        });
    });

    it('course class get by id success with participants', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/${courseClass.id}?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${STUDENTS}`,
        )
        .end((err, res) => {
          res.should.have.status(200);
          res.body.should.have.property('id');
          res.body.should.have.property('startYear');
          res.body.should.have.property('capacity');
          res.body.should.have.property('participantCount');
          res.body.should.have.property('semester');
          res.body.should.have.property('status');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('courseId');
          res.body.should.have.property('lecturerId');
          res.body.should.have.property('students');
          chai
            .expect(res.body.students[0])
            .to.have.property('id')
            .eql(student.id);
          res.body.should.have.property('courseClassDefaultSchedules');
          done();
        });
    });

    it('course class get by id not exist', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/${DUMMY_ID}?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${COURSE}&include=${LECTURER}`,
        )
        .end((err, res) => {
          res.should.have.status(NOT_EXIST_ERROR_CODE);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });

    it('course class get by id invalid request', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS}/${INVALID_ID}?status=${STATUS}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}&include=${COURSE}&include=${LECTURER}`,
        )
        .end((err, res) => {
          res.should.have.status(SERVER_ERROR_CODE);
          res.body.should.have.property('name').eql(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have.property('message');
          done();
        });
    });
  });

  describe('Creating course class data', () => {
    it('should return course class data when request body valid', (done) => {
      const requestBody = {
        courseId: course.id,
        lecturerId: lecturer.id,
        capacity: CAPACITY,
        participantCount: 20,
        semester: '2',
        status: STATUS,
        startYear: START_YEAR,
        courseClassDefaultSchedules: [
          {
            day: DayEnum.MONDAY.name,
            start: START_TIME,
            end: END_TIME,
          },
        ],
      };

      chai
        .request(server)
        .post(COURSE_CLASS_ENDPOINT)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(200);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('courseId').deep.equal(course.id);
          res.body.should.have.property('lecturerId').deep.equal(lecturer.id);
          res.body.should.have
            .property('capacity')
            .deep.equal(requestBody.capacity);
          res.body.should.have
            .property('participantCount')
            .deep.equal(requestBody.participantCount);
          res.body.should.have
            .property('semester')
            .deep.equal(requestBody.semester);
          res.body.should.have
            .property('status')
            .deep.equal(requestBody.status);
          res.body.should.have
            .property('startYear')
            .deep.equal(requestBody.startYear);
          res.body.should.have
            .property('courseClassDefaultSchedules')
            .to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return error when request body invalid (wrong status enum)', (done) => {
      const requestBody = {
        courseId: course.id,
        lecturerId: lecturer.id,
        capacity: CAPACITY,
        participantCount: 20,
        semester: '2',
        status: 'OPENED',
        startYear: START_YEAR,
      };

      chai
        .request(server)
        .post(COURSE_CLASS_ENDPOINT)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(400);
          res.body.should.have
            .property('name')
            .to.equal(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
              'invalid input value for enum "enum_CourseClasses_status": "OPENED"',
            );
          done();
        });
    });

    it('should return error when request body invalid (invalid UUID format)', (done) => {
      const invalidCourseUUID = `${course.id}a`;
      const requestBody = {
        courseId: invalidCourseUUID,
        lecturerId: lecturer.id,
        capacity: CAPACITY,
        participantCount: 20,
        semester: '2',
        status: STATUS,
        startYear: START_YEAR,
      };

      chai
        .request(server)
        .post(COURSE_CLASS_ENDPOINT)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(400);
          res.body.should.have
            .property('name')
            .to.equal(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
              `invalid input syntax for type uuid: \"${invalidCourseUUID}\"`,
            );
          done();
        });
    });

    it('should return error when foreign key doesnt exists', (done) => {
      const requestBody = {
        courseId: `${course.id.slice(0, -1)}1`,
        lecturerId: lecturer.id,
        capacity: CAPACITY,
        participantCount: 20,
        semester: '2',
        status: STATUS,
        startYear: START_YEAR,
        courseClassDefaultSchedules: [
          {
            day: DayEnum.MONDAY.name,
            start: START_TIME,
            end: END_TIME,
          },
        ],
      };

      chai
        .request(server)
        .post(COURSE_CLASS_ENDPOINT)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(400);
          res.body.should.have
            .property('name')
            .to.equal(SEQUELIZE_FOREIGN_KEY_ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
              'insert or update on table "CourseClasses" violates foreign key constraint "CourseClasses_courseId_fkey"',
            );
          done();
        });
    });
  });

  describe('Update course class', () => {
    it('should return correct course class when updating capacity, startYear, semester', (done) => {
      const request = {
        capacity: NEW_CAPACITY,
        startYear: NEW_START_YEAR,
        semester: SemesterEnum.EVEN,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE_CLASS}/${courseClass.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('courseId').to.not.equal(null);
          res.body.should.have.property('lecturerId').to.not.equal(null);
          res.body.should.have.property('capacity').deep.equal(NEW_CAPACITY);
          res.body.should.have.property('participantCount');
          res.body.should.have
            .property('semester')
            .deep.equal(SemesterEnum.EVEN);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('startYear').deep.equal(NEW_START_YEAR);
          res.body.should.have
            .property('courseClassDefaultSchedules')
            .to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return correct course class when updating course class default schedule (empty)', (done) => {
      const courseClassDefaultSchedules = [];
      const request = {
        courseClassDefaultSchedules,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE_CLASS}/${courseClass.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('courseId').to.not.equal(null);
          res.body.should.have.property('lecturerId').to.not.equal(null);
          res.body.should.have.property('capacity').to.not.equal(null);
          res.body.should.have.property('participantCount');
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have
            .property('courseClassDefaultSchedules')
            .to.deep.equal(courseClassDefaultSchedules);
          done();
        });
    });

    it('should return correct course class when updating course class default schedule', (done) => {
      const request = {
        courseClassDefaultSchedules: [
          {
            day: DayEnum.THURSDAY.name,
            start: START_TIME,
            end: END_TIME,
          },
        ],
      };

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE_CLASS}/${courseClass.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('courseId').to.not.equal(null);
          res.body.should.have.property('lecturerId').to.not.equal(null);
          res.body.should.have.property('capacity').to.not.equal(null);
          res.body.should.have.property('participantCount');
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have
            .property('courseClassDefaultSchedules')
            .to.not.equal(null);
          res.body.courseClassDefaultSchedules[0].should.have
            .property('day')
            .to.equal(DayEnum.THURSDAY.name);
          done();
        });
    });

    it('should return error when request body empty', (done) => {
      const request = {};

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE_CLASS}/${courseClass.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR_CODE);
          res.body.should.have.property('name').to.equal(ERROR);
          done();
        });
    });
  });
});
