const {
  CourseClass,
  CourseClassMeeting,
  Course,
  User,
  Student,
  Lecturer,
  StudyPlan,
  StudyPlanCourse,
} = require('../models/index');

const {
  EndpointEnum,
  CourseClassStatusEnum,
  SemesterEnum,
  StudyPlanStatusEnum,
  AttendanceStatusEnum,
} = require('../enums/index');

const { RequiredParameterUndefinedError } = require('../util/error');

let chai = require('chai');
let chaiHttp = require('chai-http');
let server = require('../index');
let should = chai.should();

chai.use(chaiHttp);

const EMAIL = 'ihsan@gmail.com';
const PASSWORD = 'abc123';
const SALT = 'salt';
const START_YEAR = 2021;
const SEMESTER = SemesterEnum.ODD;
const COURSE_CODE = 'IF2036';
const COURSE_NAME = 'Rekayasa Perangkat Lunak';
const COURSE_CREDIT = 3;

const SUCCESS_CODE = 200;
const REQUEST_ERROR = 400;

const ERROR = 'Error';
const REQUIRED_PARAM_ERROR =
  'Undefined one of these parameters: courseClassMeetingId, or studentId, or lecturerId';

const SAMPLE_LINK = 'www.steamcommunity.com/id/icancan';
const SAMPLE_TOPIC = 'Code Review';
const SAMPLE_NOTES = 'LGTM';
const SAMPLE_TIME_START = '2021-08-17T13:15:00.000Z';
const SAMPLE_TIME_END = '2021-08-17T14:55:00.000Z';

describe('Course Class Meeting Test', () => {
  let courseClass, courseClassMeeting, student, studyPlan;

  beforeEach(async () => {
    await CourseClass.destroy({ where: {} });
    await CourseClassMeeting.destroy({ where: {} });
    await Course.destroy({ where: {} });
    await User.destroy({ where: {} });
    await Lecturer.destroy({ where: {} });
    await Student.destroy({ where: {} });
    await StudyPlan.destroy({ where: {} });
    await StudyPlanCourse.destroy({ where: {} });

    const course = await Course.create({
      code: COURSE_CODE,
      name: COURSE_NAME,
      credits: COURSE_CREDIT,
    });

    let user = await User.create({
      email: EMAIL,
      hashed_password: PASSWORD,
      salt: SALT,
    });

    student = await Student.create({
      userId: user.id,
    });

    const lecturer = await Lecturer.create({
      userId: user.id,
    });

    courseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.OPEN,
      semester: SEMESTER,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    courseClassMeeting = await CourseClassMeeting.create({
      courseClassId: courseClass.id,
    });

    studyPlan = await StudyPlan.create({
      studentId: student.id,
      startYear: START_YEAR,
      semester: SemesterEnum.ODD,
      status: StudyPlanStatusEnum.FINAL,
    });

    await StudyPlanCourse.create({
      studyPlanId: studyPlan.id,
      courseClassId: courseClass.id,
    });
  });

  afterEach(() => {
    server.close();
  });

  describe('GET course class meeting', () => {
    it('should return course class meeting when studentId, semester, and year valid', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS_MEETING}/?studentId=${studyPlan.studentId}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}`,
        )
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('startTime');
          chai.expect(res.body[0]).to.have.property('endTime');
          chai.expect(res.body[0]).to.have.property('openAttendanceTime');
          chai.expect(res.body[0]).to.have.property('closeAttendanceTime');
          chai.expect(res.body[0]).to.have.property('CourseClass');
          done();
        });
    });

    it('should return course class meeting when lecturerId, semester, and year valid', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS_MEETING}/?lecturerId=${courseClass.lecturerId}&semester=${SemesterEnum.ODD}&startYear=${START_YEAR}`,
        )
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('startTime');
          chai.expect(res.body[0]).to.have.property('endTime');
          chai.expect(res.body[0]).to.have.property('openAttendanceTime');
          chai.expect(res.body[0]).to.have.property('closeAttendanceTime');
          chai.expect(res.body[0]).to.have.property('CourseClass');
          done();
        });
    });

    it('should return course class meeting when courseClassMeetingId valid', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.COURSE_CLASS_MEETING}/?courseClassMeetingId=${courseClassMeeting.id}&shouldIncludeAttendances=true`,
        )
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('startTime');
          res.body.should.have.property('endTime');
          res.body.should.have.property('openAttendanceTime');
          res.body.should.have.property('closeAttendanceTime');
          res.body.should.have.property('link');
          res.body.should.have.property('topic');
          res.body.should.have.property('notes');
          res.body.should.have
            .property('courseClassId')
            .deep.equal(courseClass.id);
          chai
            .expect(res.body['courseClassMeetingAttendances'])
            .to.be.an('array');
          done();
        });
    });

    it('should throw error when no params passed', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.COURSE_CLASS_MEETING}`)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have.property('name').to.equal(ERROR);
          res.body.should.have
            .property('message')
            .to.equal(REQUIRED_PARAM_ERROR);
          done();
        });
    });
  });

  describe('Update course class meeting', () => {
    describe('Correct path', () => {
      it('should return correct course class meeting when link valid', (done) => {
        const request = {
          link: SAMPLE_LINK,
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have.property('link').to.equal(request.link);
            done();
          });
      });

      it('should return correct course class meeting when topic valid', (done) => {
        const request = {
          topic: SAMPLE_TOPIC,
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have.property('topic').to.equal(request.topic);
            done();
          });
      });

      it('should return correct course class meeting when notes valid', (done) => {
        const request = {
          notes: SAMPLE_NOTES,
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have.property('notes').to.equal(request.notes);
            done();
          });
      });

      it('should return correct course class meeting when class time valid', (done) => {
        const request = {
          startClass: SAMPLE_TIME_START,
          endClass: SAMPLE_TIME_END,
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have
              .property('startTime')
              .to.equal(request.startClass);
            res.body.should.have.property('endTime').to.equal(request.endClass);
            done();
          });
      });

      it('should return correct course class meeting when attendance time valid', (done) => {
        const request = {
          startAttendance: SAMPLE_TIME_START,
          endAttendance: SAMPLE_TIME_END,
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have
              .property('openAttendanceTime')
              .to.equal(request.startAttendance);
            res.body.should.have
              .property('closeAttendanceTime')
              .to.equal(request.endAttendance);
            done();
          });
      });

      it('should return correct course class meeting when attendance valid', (done) => {
        const studentTemp = {
          studentId: student.id,
          status: AttendanceStatusEnum.SICK,
        };
        const request = {
          attendances: [studentTemp],
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have.property('courseClassMeetingAttendances');
            res.body.courseClassMeetingAttendances[0].should.have.property(
              'Student',
            );
            res.body.courseClassMeetingAttendances[0].Student.should.have
              .property('id')
              .to.equal(studentTemp.studentId);
            res.body.courseClassMeetingAttendances[0].should.have
              .property('status')
              .to.equal(studentTemp.status);
            done();
          });
      });
    });

    describe('Error path', () => {
      it('should return error when attendance invalid', (done) => {
        const studentTemp = {
          studentId: `${student.id.slice(0, -1)}1`,
          status: AttendanceStatusEnum.SICK,
        };
        const request = {
          courseClassMeetingAttendances: [studentTemp],
        };

        chai
          .request(server)
          .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.not.equal(null);
            res.body.should.have.property('message').to.not.equal(null);
            done();
          });
      });
    });

    it('should return RequiredParameterUndefinedError when no valid parameter passed', (done) => {
      const request = {};

      chai
        .request(server)
        .put(`${EndpointEnum.COURSE_CLASS_MEETING}/${courseClassMeeting.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have
            .property('name')
            .to.equal(new RequiredParameterUndefinedError().name);
          res.body.should.have.property('message').to.not.equal(null);
          done();
        });
    });
  });
});
