'use strict';
const chai = require('chai');
const User = require('../models/index')['User'];
const {
  EmailNotFoundError,
  PasswordDoesntMatchError,
} = require('../util/error');
const { getUser } = require('../util/db');
const { hashPasswordWithoutSalt } = require('../util/salt');

chai.use(require('chai-as-promised'));

const getUserTemplate = async () => {
  const user = {
    email: 'ihsan@gmail.com',
    password: 'i love waifu',
  };

  const { salt, hashedPassword } = await hashPasswordWithoutSalt(user.password);

  return {
    ...user,
    salt,
    hashed_password: hashedPassword,
  };
};

const userTest = async () => {
  let user = null;

  before(async () => {
    await User.destroy({
      where: {},
      force: true,
    });

    user = await getUserTemplate();

    await User.create(user);
  });

  after(async () => {
    await User.destroy({
      where: {},
      force: true,
    });
  });

  describe('When querying user for login operation', () => {
    it('should return user when email and hashed password match', async () => {
      const foundUser = await getUser(user.email, user.password);

      chai.expect(foundUser).to.exist;
    });

    it('should return user with matching email and hashed password', async () => {
      const foundUser = await getUser(user.email, user.password);

      chai.expect(foundUser.email).to.equal(user.email);
      chai.expect(foundUser.hashed_password).to.equal(user.hashed_password);
    });

    it('should return proper error when email not found', async () => {
      const gettingUnknownUser = async () => {
        await getUser('some random mail', 'some random password');
      };

      await chai
        .expect(gettingUnknownUser())
        .to.be.rejectedWith(EmailNotFoundError);
    });

    it('should return proper error when password doesnt match', async () => {
      const gettingKnownUserWithWrongPassword = async () => {
        await getUser(user.email, 'some random password');
      };

      await chai
        .expect(gettingKnownUserWithWrongPassword())
        .to.be.rejectedWith(PasswordDoesntMatchError);
    });
  });
};

describe('User util test', userTest);
