const User = require('../models/index')['User'];
const Lecturer = require('../models/index')['Lecturer'];
const SkillGroup = require('../models/index')['SkillGroup'];

const { LecturerRoleEnum } = require('../enums/index');

let chai = require('chai');
let chaiHttp = require('chai-http');
let server = require('../index');
let should = chai.should();

chai.use(chaiHttp);

let EMAIL = 'ihsan@gmail.com';
let PASSWORD = 'abc123';
let SALT = 'salt';
let DUMMY_ID = '57e19842-d712-45fd-9068-04cde6bf41f3';
let INVALID_ID = '57e19842-d712-45fd-9068-04cde6bf41f';
let SKILL_GROUP_NAME = 'Rekayasa Perangkat Lunak';
let USERNAME = 'ihsan.ma';
let FULLNAME = 'Ihsan Muhammad Asnadi';
let GENDER = 'M';
let NIP = '1234567890';
let NIK = '1234567890';
let PHONE = '0823456789';
let ACADEMIC_ROLE = LecturerRoleEnum.LECTURER;

const SINTEL_SKILL_GROUP_NAME = 'Sistem Intelijen';
const LECTURER_ENDPOINT = '/lecturer';
const { LecturerNotFoundError } = require('../util/error');

describe('Lecturer Test', () => {
  beforeEach(async () => {
    await User.destroy({ where: {} });
    await Lecturer.destroy({ where: {} });
    await SkillGroup.destroy({ where: {} });
  });

  afterEach(() => {
    server.close();
  });

  describe('/GET/:id lecturer', () => {
    let user;

    beforeEach(async () => {
      user = await User.create({
        email: EMAIL,
        hashed_password: PASSWORD,
        salt: SALT,
      });

      const lecturer = await Lecturer.create({
        userId: user.id,
      });
    });

    it('lecturer get by user id success', (done) => {
      chai
        .request(server)
        .get('/lecturer/' + user.id)
        .end((err, res) => {
          res.should.have.status(200);
          res.body.should.have.property('id');
          res.body.should.have.property('userName');
          res.body.should.have.property('fullName');
          res.body.should.have.property('gender');
          res.body.should.have.property('nip');
          res.body.should.have.property('nik');
          res.body.should.have.property('phone');
          res.body.should.have.property('academicRole');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('skillGroupId');
          res.body.should.have.property('userId');
          res.body.should.have.property('advisedStudents');
          done();
        });
    });

    it('lecturer get by user id not exist', (done) => {
      chai
        .request(server)
        .get('/lecturer/' + DUMMY_ID)
        .end((err, res) => {
          res.should.have.status(404);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });

    it('lecturer get by user id invalid uuid', (done) => {
      chai
        .request(server)
        .get('/lecturer/' + INVALID_ID)
        .end((err, res) => {
          res.should.have.status(500);
          res.body.should.have
            .property('message')
            .eql(`invalid input syntax for type uuid: "${INVALID_ID}"`);
          done();
        });
    });
  });

  describe('/PUT/:id lecturer', () => {
    let user, skillGroup;

    beforeEach(async () => {
      user = await User.create({
        email: EMAIL,
        hashed_password: PASSWORD,
        salt: SALT,
      });

      skillGroup = await SkillGroup.create({
        name: SKILL_GROUP_NAME,
      });

      await Lecturer.create({
        userId: user.id,
      });
    });

    it('lecturer update by user id success', (done) => {
      let lecturer = {
        userName: USERNAME,
        fullName: FULLNAME,
        gender: GENDER,
        nip: NIP,
        nik: NIK,
        phone: PHONE,
        academicRole: ACADEMIC_ROLE,
        skillGroupId: skillGroup.id,
      };

      chai
        .request(server)
        .put('/lecturer/' + user.id)
        .send(lecturer)
        .end((err, res) => {
          res.should.have.status(200);
          res.body.should.have.property('status').eql(200);
          res.body.should.have
            .property('message')
            .eql('Lecturer data updated.');
          done();
        });
    });

    it('lecturer update by user id not exist', (done) => {
      chai
        .request(server)
        .get('/lecturer/' + DUMMY_ID)
        .end((err, res) => {
          res.should.have.status(404);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });

    it('lecturer update by user id fail wrong payload', (done) => {
      let lecturer = {
        userName: USERNAME,
        fullName: FULLNAME,
        gender: GENDER,
        nip: NIP,
        nik: NIK,
        phone: PHONE,
        academicRole: ACADEMIC_ROLE,
        skillGroupId: DUMMY_ID,
      };

      chai
        .request(server)
        .put('/lecturer/' + user.id)
        .send(lecturer)
        .end((err, res) => {
          res.should.have.status(400);
          res.body.should.have.property('status').eql(400);
          res.body.should.have
            .property('message')
            .eql('Something wrong with your payload.');
          done();
        });
    });
  });

  describe('Get all lecturer', () => {
    let user, lecturerRPL, lecturerSintel, skillGroupRPL, skillGroupSintel;

    beforeEach(async () => {
      await User.destroy({ where: {} });
      await Lecturer.destroy({ where: {} });
      await SkillGroup.destroy({ where: {} });

      user = await User.create({
        email: EMAIL,
        hashed_password: PASSWORD,
        salt: SALT,
      });

      skillGroupRPL = await SkillGroup.create({
        name: SKILL_GROUP_NAME,
      });

      skillGroupSintel = await SkillGroup.create({
        name: SINTEL_SKILL_GROUP_NAME,
      });

      lecturerRPL = await Lecturer.create({
        userId: user.id,
        skillGroupId: skillGroupRPL.id,
      });

      lecturerSintel = await Lecturer.create({
        userId: user.id,
        skillGroupId: skillGroupSintel.id,
      });
    });

    afterEach(async () => {
      await User.destroy({ where: {} });
      await Lecturer.destroy({ where: {} });
      await SkillGroup.destroy({ where: {} });

      server.close();
    });

    it('should return all lecturer when passing no query', (done) => {
      chai
        .request(server)
        .get(LECTURER_ENDPOINT)
        .end((_, res) => {
          res.should.have.status(200);
          res.body.should.be.an('array');
          res.body.should.have.length(2);
          done();
        });
    });

    it('should return lecturer when passing 1 valid skill group', (done) => {
      chai
        .request(server)
        .get(`${LECTURER_ENDPOINT}/?skillGroups=${skillGroupRPL.id}`)
        .end((_, res) => {
          res.should.have.status(200);
          res.body.should.be.an('array');
          res.body.should.have.length(1);
          res.body[0].skillGroupId.should.deep.equal(skillGroupRPL.id);
          done();
        });
    });

    it('should return lecturer when passing (another) 1 valid skill group', (done) => {
      chai
        .request(server)
        .get(`${LECTURER_ENDPOINT}/?skillGroups=${skillGroupSintel.id}`)
        .end((_, res) => {
          res.should.have.status(200);
          res.body.should.be.an('array');
          res.body.should.have.length(1);
          res.body[0].skillGroupId.should.deep.equal(skillGroupSintel.id);
          done();
        });
    });

    it('should return all lecturer when passing more than 1 valid skill group', (done) => {
      chai
        .request(server)
        .get(
          `${LECTURER_ENDPOINT}/?skillGroups=${skillGroupSintel.id}&skillGroups=${skillGroupRPL.id}`,
        )
        .end((_, res) => {
          res.should.have.status(200);
          res.body.should.be.an('array');
          res.body.should.have.length(2);
          res.body[0].skillGroupId.should.deep.equal(skillGroupRPL.id);
          res.body[1].skillGroupId.should.deep.equal(skillGroupSintel.id);
          done();
        });
    });

    it('should throw error when no lecturer found', (done) => {
      const error = new LecturerNotFoundError();

      chai
        .request(server)
        .get(`${LECTURER_ENDPOINT}/?skillGroups=${DUMMY_ID}`)
        .end((_, res) => {
          res.should.have.status(404);
          res.body.should.have.property('name');
          res.body.should.have.property('message');
          res.body.name.should.deep.equal(error.name);
          res.body.message.should.deep.equal(error.message);
          done();
        });
    });
  });
});
