process.env.NODE_ENV='test';
const {
  Option,
  Major,
  Faculty
} = require('../models/index');

const {
  EndpointEnum,
} = require('../enums/index');

const chai = require('chai');
const chaiHttp = require('chai-http');
const server = require('../index');
const should = chai.should();

chai.use(chaiHttp);

const NAME = 'Rekayasa Perangkat Lunak';
const FACULTY_FULL_NAME = 'Fakultas Seni Rupa dan Desain';

const ERROR = 'Error';
const NOT_EXIST_MESSAGE = 'Object doesn\'t exist.';
const SEQUELIZE_DATABASE_ERROR_MESSAGE = 'SequelizeDatabaseError';

const SUCCESS_CODE = 200;
const NOT_EXIST_CODE = 404;
const INTERNAL_SERVER_ERROR_CODE = 500;

describe('Option Test', () => {
  let option;

  beforeEach(async () => {
    await Faculty.destroy({ where: {} });
    await Major.destroy({ where: {} });
    await Option.destroy({ where: {} });

    const faculty = await Faculty.create({
      fullName: FACULTY_FULL_NAME,
    });

    const major = await Major.create({
      facultyId: faculty.id,
    });

    option = await Option.create({
      name: NAME,
      majorId: major.id,
    });
  });

  afterEach(() => {
    server.close();
  });

  describe('/GET option', () => {

    it('should get all options data', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.OPTION)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('majorId', option.majorId);
          chai.expect(res.body[0]).to.have.property('name', NAME);
          done();
        });
    });

    it('should get all options data with major id', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.OPTION)
        .query({ majorId: option.majorId })
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          chai.expect(res.body).to.be.an('array');
          chai.expect(res.body[0]).to.have.property('majorId', option.majorId);
          chai.expect(res.body[0]).to.have.property('name', NAME);
          done();
        });
    });

    it('should return not exist error if no option in given major', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.OPTION)
        .query({ majorId: `${option.majorId.slice(0,-1)}A` })
        .end((err, res) => {
          res.should.have.status(NOT_EXIST_CODE);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql(NOT_EXIST_MESSAGE);
          done();
        });
    });

    it('should return error if major id invalid', (done) => {
      chai
        .request(server)
        .get(EndpointEnum.OPTION)
        .query({ majorId: option.majorId.slice(0,-1) })
        .end((err, res) => {
          res.should.have.status(INTERNAL_SERVER_ERROR_CODE);
          res.body.should.have.property('name').eql(SEQUELIZE_DATABASE_ERROR_MESSAGE);
          res.body.should.have.property('message').eql(`invalid input syntax for type uuid: "${option.majorId.slice(0,-1)}"`);
          done();
        });
    });
  });
});
