const User = require('../models/index')['User'];
const Student = require('../models/index')['Student'];
const Lecturer = require('../models/index')['Lecturer'];
const Major = require('../models/index')['Major'];
const {
  CourseClass,
  Course,
  StudyPlan,
  StudyPlanCourse,
} = require('../models/index');

const {
  EndpointEnum,
  CourseClassStatusEnum,
  SemesterEnum,
  ScoreEnum,
  ScoreStatusEnum,
} = require('../enums/index');

const { RequiredParameterUndefinedError } = require('../util/error');

let chai = require('chai');
let chaiHttp = require('chai-http');
let server = require('../index');
const should = chai.should();

chai.use(chaiHttp);

const EMAIL = 'ihsan@gmail.com';
const PASSWORD = 'abc123';
const SALT = 'salt';
const DUMMY_ID = '57e19842-d712-45fd-9068-04cde6bf41f3';
const INVALID_ID = '57e19842-d712-45fd-9068-04cde6bf41f';
const LECTURER_USERNAME = 'di.aditia';
const MAJOR_NAME = 'Informatika';
const USERNAME = 'ihsan.ma';
const FULLNAME = 'Ihsan Muhammad Asnadi';
const GENDER = 'M';
const NIM = '23520019';
const NIK = '1234567890';
const PHONE = '0823456789';
const YEAR = 2020;

const SUCCESS_CODE = 200;
const SERVER_ERROR_CODE = 500;
const NOT_EXIST_ERROR_CODE = 404;
const ERROR = 'Error';

const START_YEAR = 2021;
const SEMESTER = SemesterEnum.ODD;
const COURSE_CODE = 'IF2036';
const COURSE_NAME = 'Rekayasa Perangkat Lunak';
const COURSE_CREDIT = 3;
const NOTES = 'LGTM';

const REQUEST_ERROR = 400;

describe('Student Test', () => {
  let user,
    availableCourseClass,
    unavailableCourseClass,
    student,
    studyPlan,
    studyPlanCourse;

  beforeEach(async () => {
    await User.destroy({ where: {} });
    await Student.destroy({ where: {} });
    await CourseClass.destroy({ where: {} });
    await Course.destroy({ where: {} });
    await Lecturer.destroy({ where: {} });
    await Student.destroy({ where: {} });
    await StudyPlan.destroy({ where: {} });

    const course = await Course.create({
      code: COURSE_CODE,
      name: COURSE_NAME,
      credits: COURSE_CREDIT,
    });

    user = await User.create({
      email: EMAIL,
      hashed_password: PASSWORD,
      salt: SALT,
    });

    student = await Student.create({
      userId: user.id,
    });

    const lecturer = await Lecturer.create({
      userId: user.id,
    });

    availableCourseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.OPEN,
      semester: SEMESTER,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    unavailableCourseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.CLOSED,
      semester: SEMESTER,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    studyPlan = await StudyPlan.create({
      studentId: student.id,
      startYear: START_YEAR,
      semester: SemesterEnum.EVEN,
      notes: NOTES,
    });

    studyPlanCourse = await StudyPlanCourse.create({
      studyPlanId: studyPlan.id,
      courseClassId: availableCourseClass.id,
    });
  });

  afterEach(() => {
    server.close();
  });

  describe('/GET/:id student', () => {
    it('student get by user id success', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${user.id}`)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('userName');
          res.body.should.have.property('fullName');
          res.body.should.have.property('gender');
          res.body.should.have.property('nim');
          res.body.should.have.property('nik');
          res.body.should.have.property('phone');
          res.body.should.have.property('advisorId');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('majorId');
          res.body.should.have.property('userId');
          done();
        });
    });

    it('student get by user id not exist', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${DUMMY_ID}`)
        .end((err, res) => {
          res.should.have.status(NOT_EXIST_ERROR_CODE);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });

    it('student get by user id invalid uuid', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${INVALID_ID}`)
        .end((err, res) => {
          res.should.have.status(SERVER_ERROR_CODE);
          res.body.should.have.property('name').to.not.eql(null);
          res.body.should.have.property('message').to.not.eql(null);
          done();
        });
    });

    it('should get by user id success include historical transcript', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${user.id}?includeTranscript=true`)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('userName');
          res.body.should.have.property('fullName');
          res.body.should.have.property('gender');
          res.body.should.have.property('nim');
          res.body.should.have.property('nik');
          res.body.should.have.property('phone');
          res.body.should.have.property('studyPlans');
          res.body.should.have.property('advisorId');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('majorId');
          res.body.should.have.property('userId');
          done();
        });
    });

    it('should get by user id success include cumulative score', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${user.id}?includeScore=true`)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('userName');
          res.body.should.have.property('fullName');
          res.body.should.have.property('gender');
          res.body.should.have.property('nim');
          res.body.should.have.property('nik');
          res.body.should.have.property('phone');
          res.body.should.have.property('score').to.not.eql(null);
          res.body.score.should.have.property('nr');
          res.body.score.should.have.property('ip');
          res.body.score.should.have.property('ipk');
          res.body.should.have.property('advisorId');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('majorId');
          res.body.should.have.property('userId');
          done();
        });
    });

    it('should get by user id success include credits total', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${user.id}?includeCredits=true`)
        .end((err, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id');
          res.body.should.have.property('userName');
          res.body.should.have.property('fullName');
          res.body.should.have.property('gender');
          res.body.should.have.property('nim');
          res.body.should.have.property('nik');
          res.body.should.have.property('phone');
          res.body.should.have.property('credits').to.not.eql(null);
          res.body.should.have.property('advisorId');
          res.body.should.have.property('createdAt');
          res.body.should.have.property('updatedAt');
          res.body.should.have.property('majorId');
          res.body.should.have.property('userId');
          done();
        });
    });
  });

  describe('/PUT/:id student', () => {
    let user, lecturer, major;

    beforeEach(async () => {
      user = await User.create({
        email: EMAIL,
        hashed_password: PASSWORD,
        salt: SALT,
      });

      lecturer = await Lecturer.create({
        username: LECTURER_USERNAME,
      });

      major = await Major.create({
        name: MAJOR_NAME,
      });

      await Student.create({
        userId: user.id,
      });
    });

    it('student update by user id success', (done) => {
      let student = {
        userName: USERNAME,
        fullName: FULLNAME,
        gender: GENDER,
        nim: NIM,
        nik: NIK,
        phone: PHONE,
        year: YEAR,
        advisorId: lecturer.id,
        majorId: major.id,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDENT}/${user.id}`)
        .send(student)
        .end((err, res) => {
          res.should.have.status(200);
          res.body.should.have.property('status').eql(200);
          res.body.should.have.property('message').eql('Student data updated.');
          done();
        });
    });

    it('student update by user id not exist', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDENT}/${DUMMY_ID}`)
        .end((err, res) => {
          res.should.have.status(NOT_EXIST_ERROR_CODE);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql("Object doesn't exist.");
          done();
        });
    });

    it('student update by user id fail wrong payload', (done) => {
      let student = {
        userName: USERNAME,
        fullName: FULLNAME,
        gender: GENDER,
        nim: NIM,
        nik: NIK,
        phone: PHONE,
        year: YEAR,
        advisorId: lecturer.id,
        majorId: DUMMY_ID,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDENT}/${user.id}`)
        .send(student)
        .end((err, res) => {
          res.should.have.status(400);
          res.body.should.have.property('status').eql(400);
          res.body.should.have
            .property('message')
            .eql('Something wrong with your payload.');
          done();
        });
    });
  });

  describe('Update student score', () => {
    let error, request;

    beforeEach(() => {
      error = new RequiredParameterUndefinedError();
      request = {
        studentId: student.id,
        courseClassId: availableCourseClass.id,
        score: ScoreEnum.A.name,
      };
    });

    describe('Correct path', () => {
      it('should return correct student data when studentId, courseClassId, and score valid', (done) => {
        chai
          .request(server)
          .put(
            `${EndpointEnum.SCORE}/?studentId=${request.studentId}&courseClassId=${request.courseClassId}`,
          )
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body.should.have.property('id').to.not.equal(null);
            res.body.should.have
              .property('attendancePercentage')
              .to.not.equal(null);
            res.body.should.have.property('score').to.not.equal(null);
            res.body.should.have.property('score').to.equal(request.score);
            res.body.should.have.property('status').to.not.equal(null);
            res.body.should.have.property('courseClassId').to.not.equal(null);
            res.body.should.have.property('studyPlanId').to.not.equal(null);
            res.body.should.have.property('StudyPlan').to.not.equal(null);
            done();
          });
      });
    });

    describe('Error path', () => {
      it('should throw error when no studentId passed', (done) => {
        chai
          .request(server)
          .put(`${EndpointEnum.SCORE}/?courseClassId=${request.courseClassId}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.equal(error.name);
            res.body.should.have.property('message');
            done();
          });
      });

      it('should throw error when no courseClassId passed', (done) => {
        chai
          .request(server)
          .put(`${EndpointEnum.SCORE}/?studentId=${request.studentId}`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.equal(error.name);
            res.body.should.have.property('message');
            done();
          });
      });

      it('should throw error when no score passed', (done) => {
        const { score, ...clonedRequest } = request;

        chai
          .request(server)
          .put(
            `${EndpointEnum.SCORE}/?studentId=${request.studentId}&courseClassId=${request.courseClassId}`,
          )
          .send(clonedRequest)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.equal(error.name);
            res.body.should.have.property('message');
            done();
          });
      });
    });
  });

  describe('Update student score status', () => {
    let error, request;

    beforeEach(() => {
      error = new RequiredParameterUndefinedError();
      request = {
        courseClassId: availableCourseClass.id,
        scoreStatus: ScoreStatusEnum.FINAL,
      };
    });

    describe('Correct path', () => {
      it('should return correct study plan course data when courseClassId and scoreStatus valid', (done) => {
        chai
          .request(server)
          .put(
            `${EndpointEnum.SCORE_STATUS}/?courseClassId=${request.courseClassId}`,
          )
          .send(request)
          .end((_, res) => {
            res.should.have.status(SUCCESS_CODE);
            res.body[0].should.have.property('id').to.not.equal(null);
            res.body[0].should.have
              .property('attendancePercentage')
              .to.not.equal(null);
            res.body[0].should.have.property('score').to.not.equal(null);
            res.body[0].should.have.property('status').to.not.equal(null);
            res.body[0].should.have.property('scoreStatus').to.not.equal(null);
            res.body[0].should.have
              .property('scoreStatus')
              .to.equal(request.scoreStatus);
            res.body[0].should.have
              .property('courseClassId')
              .to.not.equal(null);
            res.body[0].should.have.property('studyPlanId').to.not.equal(null);
            done();
          });
      });
    });

    describe('Error path', () => {
      it('should throw error when no courseClassId passed', (done) => {
        chai
          .request(server)
          .put(`${EndpointEnum.SCORE_STATUS}/`)
          .send(request)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.equal(error.name);
            res.body.should.have.property('message');
            done();
          });
      });

      it('should throw error when no score status passed', (done) => {
        const { scoreStatus, ...clonedRequest } = request;

        chai
          .request(server)
          .put(`${EndpointEnum.SCORE}/?courseClassId=${request.courseClassId}`)
          .send(clonedRequest)
          .end((_, res) => {
            res.should.have.status(REQUEST_ERROR);
            res.body.should.have.property('name').to.equal(error.name);
            res.body.should.have.property('message');
            done();
          });
      });
    });
  });
});
