const {
  CourseClass,
  Course,
  User,
  Student,
  Lecturer,
  StudyPlan,
  StudyPlanCourse,
} = require('../models/index');

const {
  EndpointEnum,
  CourseClassStatusEnum,
  SemesterEnum,
  StudyPlanStatusEnum,
  StudyPlanCourseStatusEnum,
} = require('../enums/index');

let chai = require('chai');
let chaiHttp = require('chai-http');
let server = require('../index');
let should = chai.should();

chai.use(chaiHttp);

const EMAIL = 'ihsan@gmail.com';
const PASSWORD = 'abc123';
const SALT = 'salt';
const START_YEAR = 2021;
const SEMESTER = SemesterEnum.ODD;
const STUDY_PLAN_STATUS = StudyPlanStatusEnum.DRAFT;
const COURSE_CODE = 'IF2036';
const COURSE_NAME = 'Rekayasa Perangkat Lunak';
const COURSE_CREDIT = 3;
const INVALID_ID = '57e19842-d712-45fd-9068-04cde6bf41f';
const SCORE = 'T';
const NOTES = 'LGTM';

const SUCCESS_CODE = 200;
const REQUEST_ERROR = 400;
const INTERNAL_SERVER_ERROR = 500;

const SEQUELIZE_DATABASE_ERROR = 'SequelizeDatabaseError';
const INVALID_UUID_ERROR =
  'invalid input syntax for type uuid: "' + INVALID_ID + '"';
const ERROR = 'Error';
const CLASS_UNAVAILABLE_ERROR = 'Some of course classes are unavailable.';
const STUDY_PLAN_EXIST_ERROR = "Study plan's already exist";
const SEMESTER_UNDEFINED_ERROR = 'Undefined semester';

describe('Study Plan Test', () => {
  let availableCourseClass,
    availableCourseClassEven,
    unavailableCourseClass,
    student,
    studyPlan,
    studyPlanCourse;

  beforeEach(async () => {
    await CourseClass.destroy({ where: {} });
    await Course.destroy({ where: {} });
    await Lecturer.destroy({ where: {} });
    await Student.destroy({ where: {} });
    await StudyPlan.destroy({ where: {} });

    const course = await Course.create({
      code: COURSE_CODE,
      name: COURSE_NAME,
      credits: COURSE_CREDIT,
    });

    let user = await User.create({
      email: EMAIL,
      hashed_password: PASSWORD,
      salt: SALT,
    });

    student = await Student.create({
      userId: user.id,
    });

    const lecturer = await Lecturer.create({
      userId: user.id,
    });

    availableCourseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.OPEN,
      semester: SEMESTER,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    availableCourseClassEven = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.OPEN,
      semester: SemesterEnum.EVEN,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    unavailableCourseClass = await CourseClass.create({
      startYear: START_YEAR,
      status: CourseClassStatusEnum.CLOSED,
      semester: SEMESTER,
      courseId: course.id,
      lecturerId: lecturer.id,
    });

    studyPlan = await StudyPlan.create({
      studentId: student.id,
      startYear: START_YEAR,
      semester: SemesterEnum.EVEN,
      notes: NOTES,
    });

    studyPlanCourse = await StudyPlanCourse.create({
      studyPlanId: studyPlan.id,
      courseClassId: availableCourseClassEven.id,
    });
  });

  afterEach(() => {
    server.close();
  });

  describe('Creating study plan data', () => {
    it('should return study plan data when request body valid', (done) => {
      const requestBody = {
        studentId: student.id,
        startYear: START_YEAR,
        semester: SEMESTER,
        status: STUDY_PLAN_STATUS,
        notes: NOTES,
        studyPlanCourses: [
          {
            courseClassId: availableCourseClass.id,
            score: SCORE,
            attendancePercentage: 0,
          },
        ],
      };

      chai
        .request(server)
        .post(EndpointEnum.STUDY_PLAN)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('studentId').deep.equal(student.id);
          res.body.should.have
            .property('creditsTotal')
            .deep.equal(COURSE_CREDIT);
          res.body.should.have
            .property('semester')
            .deep.equal(requestBody.semester);
          res.body.should.have
            .property('status')
            .deep.equal(requestBody.status);
          res.body.should.have
            .property('startYear')
            .deep.equal(requestBody.startYear);
          res.body.should.have.property('notes').deep.equal(NOTES);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          done();
        });
    });

    it('should return error when uuid is invalid', (done) => {
      const requestBody = {
        studentId: INVALID_ID,
        startYear: START_YEAR,
        semester: SEMESTER,
        status: STUDY_PLAN_STATUS,
        studyPlanCourses: [
          {
            courseClassId: availableCourseClass.id,
            score: SCORE,
            attendancePercentage: 0,
          },
        ],
      };

      chai
        .request(server)
        .post(EndpointEnum.STUDY_PLAN)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(INTERNAL_SERVER_ERROR);
          res.body.should.have.property('name').eql(SEQUELIZE_DATABASE_ERROR);
          res.body.should.have.property('message').eql(INVALID_UUID_ERROR);
          done();
        });
    });

    it('should return error when class is unavailable', (done) => {
      const requestBody = {
        studentId: student.id,
        startYear: START_YEAR,
        semester: SEMESTER,
        status: STUDY_PLAN_STATUS,
        studyPlanCourses: [
          {
            courseClassId: unavailableCourseClass.id,
            score: SCORE,
            attendancePercentage: 0,
          },
        ],
      };

      chai
        .request(server)
        .post(EndpointEnum.STUDY_PLAN)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql(CLASS_UNAVAILABLE_ERROR);
          done();
        });
    });

    it('should return error when study plan is already exist', (done) => {
      const requestBody = {
        studentId: student.id,
        startYear: START_YEAR,
        semester: SemesterEnum.EVEN,
      };

      chai
        .request(server)
        .post(EndpointEnum.STUDY_PLAN)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have.property('message').eql(STUDY_PLAN_EXIST_ERROR);
          done();
        });
    });

    it('should return error when required parameter is undefined', (done) => {
      const requestBody = {
        studentId: student.id,
        startYear: START_YEAR,
      };

      chai
        .request(server)
        .post(EndpointEnum.STUDY_PLAN)
        .send(requestBody)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have.property('name').eql(ERROR);
          res.body.should.have
            .property('message')
            .eql(SEMESTER_UNDEFINED_ERROR);
          done();
        });
    });
  });

  describe('Getting study plan', () => {
    it('should return study plan when studentId, semester, and year valid', (done) => {
      chai
        .request(server)
        .get(
          `${EndpointEnum.STUDY_PLAN}/?studentId=${studyPlan.studentId}&semester=${SemesterEnum.EVEN}&startYear=${START_YEAR}`,
        )
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').deep.equal(START_YEAR);
          res.body.should.have.property('creditsTotal').deep.equal(0);
          res.body.should.have
            .property('semester')
            .deep.equal(SemesterEnum.EVEN);
          res.body.should.have.property('status').deep.equal(STUDY_PLAN_STATUS);
          res.body.should.have.property('notes').deep.equal(NOTES);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have
            .property('studentId')
            .deep.equal(studyPlan.studentId);
          done();
        });
    });

    it('should return study plan when studyPlanId valid', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDY_PLAN}/?studyPlanId=${studyPlan.id}`)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').deep.equal(studyPlan.id);
          res.body.should.have.property('startYear').deep.equal(START_YEAR);
          res.body.should.have.property('creditsTotal').deep.equal(0);
          res.body.should.have
            .property('semester')
            .deep.equal(SemesterEnum.EVEN);
          res.body.should.have.property('status').deep.equal(STUDY_PLAN_STATUS);
          res.body.should.have.property('notes').deep.equal(NOTES);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          done();
        });
    });

    it('should throw error when no params passed', (done) => {
      chai
        .request(server)
        .get(`${EndpointEnum.STUDY_PLAN}`)
        .end((_, res) => {
          res.should.have.status(REQUEST_ERROR);
          res.body.should.have.property('name').to.equal(ERROR);
          res.body.should.have
            .property('message')
            .to.equal(
              'Undefined one of these parameters: studentId, or studyPlanId',
            );
          done();
        });
    });
  });

  describe('Update study plan', () => {
    it('should return correct study plan when updating notes', (done) => {
      const newNotes = 'So far so good, Dimas said';
      const request = {
        notes: newNotes,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDY_PLAN}/${studyPlan.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have.property('creditsTotal').to.not.equal(null);
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('notes').to.not.equal(null);
          res.body.should.have.property('notes').to.not.equal(NOTES);
          res.body.should.have.property('notes').to.equal(newNotes);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          done();
        });
    });

    it('should return correct study plan when updating status', (done) => {
      const newStatus = StudyPlanStatusEnum.FINAL;
      const request = {
        status: newStatus,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDY_PLAN}/${studyPlan.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have.property('creditsTotal').to.not.equal(null);
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have
            .property('status')
            .to.not.equal(STUDY_PLAN_STATUS);
          res.body.should.have.property('status').to.equal(newStatus);
          res.body.should.have.property('notes').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          done();
        });
    });

    it('should return correct study plan when updating study plan courses (empty)', (done) => {
      const studyPlanCourses = [];
      const request = {
        studyPlanCourses,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDY_PLAN}/${studyPlan.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have.property('creditsTotal').to.not.equal(null);
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('notes').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          res.body.should.have
            .property('studyPlanCourses')
            .to.deep.equal(studyPlanCourses);
          done();
        });
    });

    it('should return correct study plan when updating study plan courses (set APPROVED)', (done) => {
      studyPlanCourse.status = StudyPlanCourseStatusEnum.APPROVED;

      const studyPlanCourses = [studyPlanCourse];

      const request = {
        studyPlanCourses,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDY_PLAN}/${studyPlan.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have.property('creditsTotal').to.not.equal(null);
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('notes').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          res.body.studyPlanCourses[0].should.have
            .property('id')
            .to.equal(studyPlanCourse.id);
          res.body.studyPlanCourses[0].should.have
            .property('status')
            .to.equal(StudyPlanCourseStatusEnum.APPROVED);
          done();
        });
    });

    it('should return correct study plan when updating study plan courses (set UNAPPROVED)', (done) => {
      studyPlanCourse.status = StudyPlanCourseStatusEnum.UNAPPROVED;

      const studyPlanCourses = [studyPlanCourse];

      const request = {
        studyPlanCourses,
      };

      chai
        .request(server)
        .put(`${EndpointEnum.STUDY_PLAN}/${studyPlan.id}`)
        .send(request)
        .end((_, res) => {
          res.should.have.status(SUCCESS_CODE);
          res.body.should.have.property('id').to.not.equal(null);
          res.body.should.have.property('startYear').to.not.equal(null);
          res.body.should.have.property('creditsTotal').to.not.equal(null);
          res.body.should.have.property('semester').to.not.equal(null);
          res.body.should.have.property('status').to.not.equal(null);
          res.body.should.have.property('notes').to.not.equal(null);
          res.body.should.have.property('updatedAt').to.not.equal(null);
          res.body.should.have.property('createdAt').to.not.equal(null);
          res.body.should.have.property('studentId').to.not.equal(null);
          res.body.should.have.property('studyPlanCourses').to.not.equal(null);
          res.body.studyPlanCourses[0].should.have
            .property('id')
            .to.equal(studyPlanCourse.id);
          res.body.studyPlanCourses[0].should.have
            .property('status')
            .to.equal(StudyPlanCourseStatusEnum.UNAPPROVED);
          done();
        });
    });
  });
});
