'use strict';
const {
  CourseClassMeeting,
  StudyPlanCourse,
  CourseClassMeetingAttendance,
  CourseClassDefaultSchedule,
} = require('../models/index');

const {
  DayEnum,
  SemesterEnum,
  CourseClassStatusEnum,
  AttendanceStatusEnum,
} = require('../enums/index');

const { RequiredParameterUndefinedError } = require('./error');

const handleRequestWithInvalidRequestBody = (res, error) => {
  const { name, message } = error;
  console.error(error);

  res.status(400).json({
    name,
    message,
  });
};

const checkRequiredParameter = (requiredReq) => {
  const { oneOf } = requiredReq;

  if (oneOf) {
    let requirementFulfilled = false;

    for (const param in oneOf) {
      if (oneOf[param]) {
        requirementFulfilled = true;
        break;
      }
    }

    if (!requirementFulfilled) {
      const message = `one of these parameters: ${Object.keys(oneOf).join(
        ', or ',
      )}`;
      throw new RequiredParameterUndefinedError(message);
    }
  } else {
    let error = [];
    for (const param in requiredReq) {
      if (!requiredReq[param]) error.push(param);
    }

    if (error.length !== 0)
      throw new RequiredParameterUndefinedError(error.join(','));
  }
};

const handleRequestWithInternalServerError = (res, error) => {
  const { name, message } = error;
  console.error(error);

  res.status(500).json({
    name,
    message,
  });
};

const handleRequestWithResourceItemNotFound = (res, error) => {
  const { name, message } = error;
  console.error(error);

  res.status(404).json({
    name,
    message,
  });
};

const generateCourseClassMeetings = async (courseClass) => {
  if (courseClass.status !== CourseClassStatusEnum.OPEN) {
    return;
  }

  let startDate, endDate;
  if (courseClass.semester === SemesterEnum.ODD) {
    startDate = new Date(`${courseClass.startYear}-08-15T00:00:00.000+07:00`);
    endDate = new Date(`${courseClass.startYear}-12-15T00:00:00.000+07:00`);
  } else {
    startDate = new Date(
      `${courseClass.startYear + 1}-01-15T00:00:00.000+07:00`,
    );
    endDate = new Date(`${courseClass.startYear + 1}-05-15T00:00:00.000+07:00`);
  }

  const courseClassDefaultSchedules = await CourseClassDefaultSchedule.findAll({
    where: {
      courseClassId: courseClass.id,
    },
  });

  for (let i = 0; i < courseClassDefaultSchedules.length; i++) {
    for (
      let d = new Date(startDate.getTime());
      d <= endDate;
      d.setDate(d.getDate() + 1)
    ) {
      if (d.getDay() === DayEnum[courseClassDefaultSchedules[i].day].value) {
        const startTimeParts = courseClassDefaultSchedules[i].start.match(
          /(\d+):(\d+):(\d+)/,
        );
        const endTimeParts = courseClassDefaultSchedules[i].end.match(
          /(\d+):(\d+):(\d+)/,
        );

        let start = new Date(d.getTime());
        start.setHours(parseInt(startTimeParts[1], 10));
        start.setMinutes(parseInt(startTimeParts[2], 10));
        start.setSeconds(parseInt(startTimeParts[3], 10));

        let end = new Date(d.getTime());
        end.setHours(parseInt(endTimeParts[1], 10));
        end.setMinutes(parseInt(endTimeParts[2], 10));
        end.setSeconds(parseInt(endTimeParts[3], 10));

        await CourseClassMeeting.create({
          courseClassId: courseClass.id,
          startTime: start,
          endTime: end,
          openAttendanceTime: start,
          closeAttendanceTime: end,
        });
      }
    }
  }
};

//Used when studyPlan.status is updated to FINAL
const generateCourseClassMeetingAttendances = async (studyPlan) => {
  const studyPlanCourses = await StudyPlanCourse.findAll({
    where: {
      studyPlanId: studyPlan.id,
    },
  });

  for (let i = 0; i < studyPlanCourses.length; i++) {
    const meetings = await CourseClassMeeting.findAll({
      where: {
        courseClassId: studyPlanCourses[i].courseClassId,
      },
    });

    for (let j = 0; j < meetings.length; j++) {
      await CourseClassMeetingAttendance.create({
        courseClassMeetingId: meetings[j].dataValues.id,
        studentId: studyPlan.studentId,
        status: AttendanceStatusEnum.ABSENT,
      });
    }
  }
};

module.exports = {
  handleRequestWithInvalidRequestBody,
  handleRequestWithInternalServerError,
  handleRequestWithResourceItemNotFound,
  checkRequiredParameter,
  generateCourseClassMeetings,
  generateCourseClassMeetingAttendances,
};
