'use strict';
const { Op } = require('sequelize');
const {
  CourseClassMeeting,
  CourseClass,
  Course,
  CourseClassMeetingAttendance,
  Student,
} = require('../../models/index');

const { getStudyPlan } = require('./study-plan');
const { getCourseClass } = require('./course-class');

const { NotExistError } = require('../error');

const getCourseClassMeetingById = async (opts = {}) => {
  const { shouldIncludeAttendances } = opts;

  if (shouldIncludeAttendances) {
    opts.include = [
      {
        model: CourseClassMeetingAttendance,
        as: 'courseClassMeetingAttendances',
        attributes: ['status'],
        include: Student,
      },
    ];
  }

  const courseClassMeeting = await CourseClassMeeting.findOne(opts);

  if (!courseClassMeeting) throw new NotExistError();

  return courseClassMeeting;
};

const getCourseClassMeetingByStudentId = async (opts = {}) => {
  const studyPlan = await getStudyPlan(opts);

  const { studyPlanCourses } = studyPlan;

  const courseClassIds = studyPlanCourses.map(
    (studyPlanCourse) => studyPlanCourse.courseClassId,
  );

  return await getCourseClassMeetingByCourseClassIds(courseClassIds);
};

const getCourseClassMeetingByLecturerId = async (opts = {}) => {
  const courseClasses = await getCourseClass({ body: { opts } }, true);

  const courseClassIds = courseClasses.map((courseClass) => courseClass.id);

  return await getCourseClassMeetingByCourseClassIds(courseClassIds);
};

const getCourseClassMeetingByCourseClassIds = async (courseClassIds) => {
  if (!courseClassIds.length) return [];
  try {
    const courseClassMeetings = await CourseClassMeeting.findAll({
      where: {
        courseClassId: {
          [Op.or]: courseClassIds,
        },
      },
      attributes: [
        'id',
        'startTime',
        'endTime',
        'openAttendanceTime',
        'closeAttendanceTime',
      ],
      include: [
        {
          model: CourseClass,
          attributes: ['courseId'],
          include: [
            {
              model: Course,
              attributes: ['code', 'name'],
            },
          ],
        },
      ],
    });

    if (!courseClassMeetings) throw new NotExistError();

    return courseClassMeetings;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

const updateCourseClassMeeting = async (courseClassMeeting, changeset) => {
  const { courseClassMeetingAttendances } = changeset;

  if (courseClassMeetingAttendances) {
    await deleteAllAttendance(courseClassMeeting.id);
    await recreateAllCourseClassMeetingAttendance(
      courseClassMeeting.id,
      courseClassMeetingAttendances,
    );
  }

  const result = await courseClassMeeting.update(changeset);

  await result.reload();

  return result;
};

const deleteAllAttendance = async (courseClassMeetingId) => {
  await CourseClassMeetingAttendance.destroy({
    where: {
      courseClassMeetingId,
    },
  });
};

const recreateAllCourseClassMeetingAttendance = async (
  courseClassMeetingId,
  courseClassMeetingAttendances,
) => {
  for (const attendance of courseClassMeetingAttendances) {
    await CourseClassMeetingAttendance.create({
      courseClassMeetingId,
      ...attendance,
    });
  }
};

module.exports = {
  getCourseClassMeetingById,
  getCourseClassMeetingByStudentId,
  getCourseClassMeetingByLecturerId,
  updateCourseClassMeeting,
};
