const {
  StudyPlan,
  StudyPlanCourse,
  Student,
  CourseClass,
  Course,
} = require('../../models/index');

const { ScoreEnum, ScoreStatusEnum } = require('../../enums/index');

const { NotExistError } = require('../error');

const getStudentIPK = async (student) => {
  let cumulativeTotal = 0;
  let cumulativeCount = 0;

  let studyPlans = student.studyPlans;

  if (!studyPlans) {
    studyPlans = await getHistoricalTranscript(student.id);
  }

  for (const studyPlan of studyPlans) {
    const { count, total } = getScoreOneStudyPlan(studyPlan);
    cumulativeCount += count;
    cumulativeTotal += total;
  }

  if (cumulativeCount !== 0)
    return Number((cumulativeTotal / cumulativeCount).toFixed(2));

  return 0;
};

const getScoreOneStudyPlan = (studyPlan) => {
  let count = 0;
  let total = 0;

  for (const studyPlanCourse of studyPlan.studyPlanCourses) {
    if (
      studyPlanCourse.score !== ScoreEnum.T.name &&
      studyPlanCourse.scoreStatus === ScoreStatusEnum.FINAL
    ) {
      count += 1;
      total += ScoreEnum[studyPlanCourse.score].value;
    }
  }

  return { count, total };
};

const getStudentIP = async (student) => {
  let total = 0;
  let count = 0;

  let studyPlans = student.studyPlans;

  if (!studyPlans) {
    studyPlans = await getHistoricalTranscript(student.id);
  }

  let courses = [];

  for (const studyPlan of studyPlans) {
    for (const studyPlanCourse of studyPlan.studyPlanCourses) {
      if (
        studyPlanCourse.score !== ScoreEnum.T.name &&
        !courses.includes(studyPlanCourse['CourseClass'].courseId) &&
        studyPlanCourse.scoreStatus === ScoreStatusEnum.FINAL
      ) {
        count += 1;
        total += ScoreEnum[studyPlanCourse.score].value;
        courses.push(studyPlanCourse['CourseClass'].courseId);
      }
    }
  }

  if (count !== 0) return Number((total / count).toFixed(2));

  return 0;
};

const getStudentNR = async (student) => {
  let nr = {};
  let studyPlans = student.studyPlans;

  if (!studyPlans) {
    studyPlans = await getHistoricalTranscript(student.id);
  }

  if (studyPlans.length !== 0) {
    for (const currentStudyPlan of studyPlans){
      const { count, total } = getScoreOneStudyPlan(currentStudyPlan);

      if (!nr[currentStudyPlan.startYear]){
        nr[currentStudyPlan.startYear] = {};
      }

      if (count !== 0) {
        nr[currentStudyPlan.startYear][currentStudyPlan.semester] = Number((total / count).toFixed(2));
      } else {
        nr[currentStudyPlan.startYear][currentStudyPlan.semester] = 0;
      }
    }
  }

  return nr;
};

const getCredits = async (student) => {
  let passed = 0;
  let scoreT = 0;
  let studyPlans = student.studyPlans;

  if (!studyPlans) {
    studyPlans = await getHistoricalTranscript(student.id);
  }

  for (const studyPlan of studyPlans) {
    for (const studyPlanCourse of studyPlan.studyPlanCourses){
      if (ScoreEnum[studyPlanCourse.score].value >= ScoreEnum.C.value && studyPlanCourse.scoreStatus === ScoreStatusEnum.FINAL){
        passed += studyPlanCourse.CourseClass.Course.credits;
      }
      if (ScoreEnum[studyPlanCourse.score].name === ScoreEnum.T.name || studyPlanCourse.scoreStatus !== ScoreStatusEnum.FINAL){
        scoreT += studyPlanCourse.CourseClass.Course.credits;
      }
    }
  }

  return {passed, scoreT};
};

const getStudent = async (opts) => {
  const student = await Student.findOne(opts);

  if (!student) throw new NotExistError();

  return student;
};

const getHistoricalTranscript = async (studentId) => {
  return await StudyPlan.findAll({
    where: {
      studentId: studentId,
    },
    include: [
      {
        model: StudyPlanCourse,
        as: 'studyPlanCourses',
        include: [
          {
            model: CourseClass,
            attributes: ['courseId'],
            include: [{
              model: Course
            }]
          },
        ],
      },
    ],
    order: [
      ['startYear', 'DESC'],
      ['semester', 'DESC'],
    ],
  });
};

module.exports = {
  getStudentIPK,
  getStudentIP,
  getStudentNR,
  getCredits,
  getStudent,
  getHistoricalTranscript,
};
