'use strict';
const {
  StudyPlan,
  StudyPlanCourse,
  Course,
  CourseClass,
} = require('../../models/index');

const {
  NotExistError,
  StudyPlanExistError,
} = require('../error');

const getStudyPlan = async (opts = {}) => {
  const studyPlan = await StudyPlan.findOne(opts);

  if (!studyPlan) throw new NotExistError();

  return studyPlan;
};

const createStudyPlan = async (data) => {
  try {
    const { studentId, semester, startYear } = data;
    const uniqueData = { studentId, semester, startYear };

    const [studyPlan, created] = await StudyPlan.findOrCreate({
      where: uniqueData,
      defaults: data,
      include: [
        {
          model: StudyPlanCourse,
          as: 'studyPlanCourses',
        },
      ],
    });

    if (!created) throw new StudyPlanExistError();

    if (studyPlan) return studyPlan;
    else throw new NotExistError();
  } catch (error) {
    throw error;
  }
};

const countTotalCredits = async (courses) => {
  let total = 0;
  for (const studyPlanCourse of courses) {
    const courseClass = await CourseClass.findOne({
      where: { id: studyPlanCourse.courseClassId },
      include: Course,
    });
    total += courseClass.Course.credits;
  }
  return total;
};

const updateStudyPlan = async (studyPlan, changeset) => {
  const { studyPlanCourses } = changeset;

  if (studyPlanCourses) {
    await deleteAllStudyPlanCourses(studyPlan.id);
    await recreateAllStudyPlanCourses(studyPlan.id, studyPlanCourses);
  }

  const result = await studyPlan.update(changeset);

  await result.reload();

  return result;
};

const deleteAllStudyPlanCourses = async (studyPlanId) => {
  await StudyPlanCourse.destroy({
    where: {
      studyPlanId,
    },
  });
};

const recreateAllStudyPlanCourses = async (studyPlanId, studyPlanCourses) => {
  for (const course of studyPlanCourses) {
    await StudyPlanCourse.create({
      studyPlanId,
      ...course,
    });
  }
};

const getStudyPlanCourse = async (opts = {}) => {
  const studyPlanCourse = await StudyPlanCourse.findOne(opts);

  if (!studyPlanCourse) throw new NotExistError();

  return studyPlanCourse;
};

const getBulkStudyPlanCourses = async (opts = {}) => {
  const studyPlanCourses = await StudyPlanCourse.findAll(opts);

  if (!studyPlanCourses) throw new NotExistError();

  return studyPlanCourses;
};

const updateStudyPlanCourse = async (studyPlanCourse, changeset) => {
  const result = await studyPlanCourse.update(changeset);

  await result.reload();

  return result;
};

module.exports = {
  getStudyPlan,
  createStudyPlan,
  countTotalCredits,
  updateStudyPlan,
  getStudyPlanCourse,
  updateStudyPlanCourse,
  getBulkStudyPlanCourses,
};
